@@ -13,9 +13,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import netaddr
 from neutron.db import common_db_mixin as base_db
+from neutron_lib.api import validators
 from neutron_lib import constants as nl_constants
 from neutron_lib.db import model_base
+from neutron_lib.exceptions import firewall_v2 as f_exc
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import uuidutils
@@ -24,10 +27,9 @@ from sqlalchemy.ext.orderinglist import ordering_list
 from sqlalchemy import orm
 from sqlalchemy.orm import exc
 
-import netaddr
-
 from neutron_fwaas.extensions import firewall_v2 as fw_ext
 
+
 LOG = logging.getLogger(__name__)
 
 
@@ -42,7 +44,7 @@ class HasDescription(object):
 class FirewallRuleV2(model_base.BASEV2, model_base.HasId, HasName,
                      HasDescription, model_base.HasProject):
     __tablename__ = "firewall_rules_v2"
-    public = sa.Column(sa.Boolean)
+    shared = sa.Column(sa.Boolean)
     protocol = sa.Column(sa.String(40))
     ip_version = sa.Column(sa.Integer)
     source_ip_address = sa.Column(sa.String(46))
@@ -65,7 +67,6 @@ class FirewallGroup(model_base.BASEV2, model_base.HasId, HasName,
                             cascade='all, delete'))
     name = sa.Column(sa.String(255))
     description = sa.Column(sa.String(1024))
-    public = sa.Column(sa.Boolean)
     ingress_firewall_policy_id = sa.Column(sa.String(36),
                                            sa.ForeignKey(
                                                'firewall_policies_v2.id'))
@@ -74,6 +75,7 @@ class FirewallGroup(model_base.BASEV2, model_base.HasId, HasName,
                                               'firewall_policies_v2.id'))
     admin_state_up = sa.Column(sa.Boolean)
     status = sa.Column(sa.String(16))
+    shared = sa.Column(sa.Boolean)
 
 
 class FirewallGroupPortAssociation(model_base.BASEV2):
@@ -109,7 +111,6 @@ class FirewallPolicy(model_base.BASEV2, model_base.HasId, HasName,
     __tablename__ = 'firewall_policies_v2'
     name = sa.Column(sa.String(255))
     description = sa.Column(sa.String(1024))
-    public = sa.Column(sa.Boolean)
     rule_count = sa.Column(sa.Integer)
     audited = sa.Column(sa.Boolean)
     rule_associations = orm.relationship(
@@ -117,6 +118,7 @@ class FirewallPolicy(model_base.BASEV2, model_base.HasId, HasName,
         backref=orm.backref('firewall_policies_v2', cascade='all, delete'),
         order_by='FirewallPolicyRuleAssociation.position',
         collection_class=ordering_list('position', count_from=1))
+    shared = sa.Column(sa.Boolean)
 
 
 class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
@@ -125,19 +127,19 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
         try:
             return self._get_by_id(context, FirewallGroup, id)
         except exc.NoResultFound:
-            raise fw_ext.FirewallGroupNotFound(firewall_id=id)
+            raise f_exc.FirewallGroupNotFound(firewall_id=id)
 
     def _get_firewall_policy(self, context, id):
         try:
             return self._get_by_id(context, FirewallPolicy, id)
         except exc.NoResultFound:
-            raise fw_ext.FirewallPolicyNotFound(firewall_policy_id=id)
+            raise f_exc.FirewallPolicyNotFound(firewall_policy_id=id)
 
     def _get_firewall_rule(self, context, id):
         try:
             return self._get_by_id(context, FirewallRuleV2, id)
         except exc.NoResultFound:
-            raise fw_ext.FirewallRuleNotFound(firewall_rule_id=id)
+            raise f_exc.FirewallRuleNotFound(firewall_rule_id=id)
 
     def _validate_fwr_protocol_parameters(self, fwr, fwr_db=None):
         protocol = fwr.get('protocol', None)
@@ -147,7 +149,7 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
                             nl_constants.PROTO_NAME_UDP):
             if (fwr.get('source_port', None) or
                     fwr.get('destination_port', None)):
-                raise fw_ext.FirewallRuleInvalidICMPParameter(
+                raise f_exc.FirewallRuleInvalidICMPParameter(
                     param="Source, destination port")
 
     def _validate_fwr_src_dst_ip_version(self, fwr, fwr_db=None):
@@ -162,12 +164,12 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
             rule_ip_version = fwr_db.ip_version
         if ((src_version and src_version != rule_ip_version) or
                 (dst_version and dst_version != rule_ip_version)):
-            raise fw_ext.FirewallIpAddressConflict()
+            raise f_exc.FirewallIpAddressConflict()
 
     def _validate_fwr_port_range(self, min_port, max_port):
         if int(min_port) > int(max_port):
             port_range = '%s:%s' % (min_port, max_port)
-            raise fw_ext.FirewallRuleInvalidPortValue(port=port_range)
+            raise f_exc.FirewallRuleInvalidPortValue(port=port_range)
 
     def _get_min_max_ports_from_range(self, port_range):
         if not port_range:
@@ -197,7 +199,6 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
                'tenant_id': firewall_rule['tenant_id'],
                'name': firewall_rule['name'],
                'description': firewall_rule['description'],
-               'public': firewall_rule['public'],
                'protocol': firewall_rule['protocol'],
                'ip_version': firewall_rule['ip_version'],
                'source_ip_address': firewall_rule['source_ip_address'],
@@ -206,7 +207,8 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
                'source_port': src_port_range,
                'destination_port': dst_port_range,
                'action': firewall_rule['action'],
-               'enabled': firewall_rule['enabled']}
+               'enabled': firewall_rule['enabled'],
+               'shared': firewall_rule['shared']}
         return self._fields(res, fields)
 
     def _make_firewall_policy_dict(self, firewall_policy, fields=None):
@@ -217,9 +219,9 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
                'tenant_id': firewall_policy['tenant_id'],
                'name': firewall_policy['name'],
                'description': firewall_policy['description'],
-               'public': firewall_policy['public'],
                'audited': firewall_policy['audited'],
-               'firewall_rules': fw_rules}
+               'firewall_rules': fw_rules,
+               'shared': firewall_policy['shared']}
         return self._fields(res, fields)
 
     def _make_firewall_group_dict(self, firewall_group, fields=None):
@@ -230,14 +232,14 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
                'tenant_id': firewall_group['tenant_id'],
                'name': firewall_group['name'],
                'description': firewall_group['description'],
-               'public': firewall_group['public'],
                'ingress_firewall_policy_id':
                    firewall_group['ingress_firewall_policy_id'],
                'egress_firewall_policy_id':
                    firewall_group['egress_firewall_policy_id'],
                'admin_state_up': firewall_group['admin_state_up'],
                'ports': fwg_ports,
-               'status': firewall_group['status']}
+               'status': firewall_group['status'],
+               'shared': firewall_group['shared']}
         return self._fields(res, fields)
 
     def _get_policy_ordered_rules(self, context, policy_id):
@@ -265,11 +267,11 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
         return firewall_group
 
     def _check_firewall_rule_conflict(self, fwr_db, fwp_db):
-        if not fwr_db['public']:
+        if not fwr_db['shared']:
             if fwr_db['tenant_id'] != fwp_db['tenant_id']:
-                raise fw_ext.FirewallRuleConflict(
+                raise f_exc.FirewallRuleConflict(
                     firewall_rule_id=fwr_db['id'],
-                    tenant_id=fwr_db['tenant_id'])
+                    project_id=fwr_db['tenant_id'])
 
     def _process_rule_for_policy(self, context, firewall_policy_id,
                                  firewall_rule_id, position, association_db):
@@ -305,7 +307,7 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
         try:
             self._get_policy_rule_association_query(
                 context, firewall_policy_id, firewall_rule_id).one()
-            raise fw_ext.FirewallRuleAlreadyAssociated(
+            raise f_exc.FirewallRuleAlreadyAssociated(
                 firewall_rule_id=firewall_rule_id,
                 firewall_policy_id=firewall_policy_id)
         except exc.NoResultFound:
@@ -320,7 +322,7 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
             return self._get_policy_rule_association_query(
                 context, firewall_policy_id, firewall_rule_id).one()
         except exc.NoResultFound:
-            raise fw_ext.FirewallRuleNotAssociatedWithPolicy(
+            raise f_exc.FirewallRuleNotAssociatedWithPolicy(
                 firewall_rule_id=firewall_rule_id,
                 firewall_policy_id=firewall_policy_id)
 
@@ -331,7 +333,7 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
         self._validate_fwr_src_dst_ip_version(fwr)
         if not fwr['protocol'] and (fwr['source_port'] or
            fwr['destination_port']):
-            raise fw_ext.FirewallRuleWithPortWithoutProtocolInvalid()
+            raise f_exc.FirewallRuleWithPortWithoutProtocolInvalid()
         src_port_min, src_port_max = self._get_min_max_ports_from_range(
             fwr['source_port'])
         dst_port_min, dst_port_max = self._get_min_max_ports_from_range(
@@ -342,7 +344,6 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
                 tenant_id=fwr['tenant_id'],
                 name=fwr['name'],
                 description=fwr['description'],
-                public=fwr['public'],
                 protocol=fwr['protocol'],
                 ip_version=fwr['ip_version'],
                 source_ip_address=fwr['source_ip_address'],
@@ -352,7 +353,8 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
                 destination_port_range_min=dst_port_min,
                 destination_port_range_max=dst_port_max,
                 action=fwr['action'],
-                enabled=fwr['enabled'])
+                enabled=fwr['enabled'],
+                shared=fwr['shared'])
             context.session.add(fwr_db)
         return self._make_firewall_rule_dict(fwr_db)
 
@@ -382,7 +384,7 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
                 dport = fwr.get('destination_port_range_min',
                                 fwr_db['destination_port_range_min'])
                 if sport or dport:
-                    raise fw_ext.FirewallRuleWithPortWithoutProtocolInvalid()
+                    raise f_exc.FirewallRuleWithPortWithoutProtocolInvalid()
             fwr_db.update(fwr)
             # if the rule on a policy, fix audited flag
             fwp_ids = self._get_policies_with_rule(context, id)
@@ -397,19 +399,17 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
             fwr = self._get_firewall_rule(context, id)
             # make sure rule is not associated with any policy
             if self._get_policies_with_rule(context, id):
-                raise fw_ext.FirewallRuleInUse(firewall_rule_id=id)
+                raise f_exc.FirewallRuleInUse(firewall_rule_id=id)
             context.session.delete(fwr)
 
     def insert_rule(self, context, id, rule_info):
         LOG.debug("insert_rule() called")
-        self._validate_insert_remove_rule_request(id, rule_info)
+        self._validate_insert_remove_rule_request(rule_info)
         firewall_rule_id = rule_info['firewall_rule_id']
         # ensure rule is not already assigned to the policy
         self._ensure_rule_not_already_associated(context, id, firewall_rule_id)
         insert_before = True
         ref_firewall_rule_id = None
-        if not firewall_rule_id:
-            raise fw_ext.FirewallRuleNotFound(firewall_rule_id=None)
         if 'insert_before' in rule_info:
             ref_firewall_rule_id = rule_info['insert_before']
         if not ref_firewall_rule_id and 'insert_after' in rule_info:
@@ -444,10 +444,8 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
 
     def remove_rule(self, context, id, rule_info):
         LOG.debug("remove_rule() called")
-        self._validate_insert_remove_rule_request(id, rule_info)
+        self._validate_insert_remove_rule_request(rule_info)
         firewall_rule_id = rule_info['firewall_rule_id']
-        if not firewall_rule_id:
-            raise fw_ext.FirewallRuleNotFound(firewall_rule_id=None)
         with context.session.begin(subtransactions=True):
             self._get_firewall_rule(context, firewall_rule_id)
             fwpra_db = self._get_policy_rule_association(context, id,
@@ -466,9 +464,18 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
                                     self._make_firewall_rule_dict,
                                     filters=filters, fields=fields)
 
-    def _validate_insert_remove_rule_request(self, id, rule_info):
+    def _validate_insert_remove_rule_request(self, rule_info):
+        """Validate rule_info dict
+
+        Check that all mandatory fields are present, otherwise raise
+        proper exception.
+        """
         if not rule_info or 'firewall_rule_id' not in rule_info:
-            raise fw_ext.FirewallRuleInfoMissing()
+            raise f_exc.FirewallRuleInfoMissing()
+        # Validator doesn't return anything if the check passes
+        if validators.validate_uuid(rule_info['firewall_rule_id']):
+            raise f_exc.FirewallRuleNotFound(
+                firewall_rule_id=rule_info['firewall_rule_id'])
 
     def _delete_rules_in_policy(self, context, firewall_policy_id):
         """Delete the rules in the  firewall policy."""
@@ -522,35 +529,35 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
         for fwrule_id in rule_id_list:
             if fwrule_id not in rules_dict:
                 # Bail as soon as we find an invalid rule.
-                raise fw_ext.FirewallRuleNotFound(
+                raise f_exc.FirewallRuleNotFound(
                     firewall_rule_id=fwrule_id)
-            if 'public' in fwp:
-                if fwp['public'] and not rules_dict[fwrule_id]['public']:
-                    raise fw_ext.FirewallRuleSharingConflict(
+            if 'shared' in fwp:
+                if fwp['shared'] and not rules_dict[fwrule_id]['shared']:
+                    raise f_exc.FirewallRuleSharingConflict(
                         firewall_rule_id=fwrule_id,
                         firewall_policy_id=fwp_db['id'])
-            elif fwp_db['public'] and not rules_dict[fwrule_id]['public']:
-                raise fw_ext.FirewallRuleSharingConflict(
+            elif fwp_db['shared'] and not rules_dict[fwrule_id]['shared']:
+                raise f_exc.FirewallRuleSharingConflict(
                     firewall_rule_id=fwrule_id,
                     firewall_policy_id=fwp_db['id'])
             else:
-                # the policy is not public, the rule and policy should be in
-                # the same project if the rule is not public.
-                if not rules_dict[fwrule_id]['public']:
-                    if (rules_dict[fwrule_id]['tenant_id'] !=
-                        fwp_db['tenant_id']):
-                        raise fw_ext.FirewallRuleConflict(
+                # the policy is not shared, the rule and policy should be in
+                # the same project if the rule is not shared.
+                if not rules_dict[fwrule_id]['shared']:
+                    if (rules_dict[fwrule_id]['tenant_id'] != fwp_db[
+                            'tenant_id']):
+                        raise f_exc.FirewallRuleConflict(
                             firewall_rule_id=fwrule_id,
-                            tenant_id=rules_dict[fwrule_id]['tenant_id'])
+                            project_id=rules_dict[fwrule_id]['tenant_id'])
 
-    def _check_if_rules_public_for_policy_public(self, context, fwp_db, fwp):
-        if fwp['public']:
+    def _check_if_rules_shared_for_policy_shared(self, context, fwp_db, fwp):
+        if fwp['shared']:
             rules_in_db = fwp_db.rule_associations
             for entry in rules_in_db:
                 fwr_db = self._get_firewall_rule(context,
                                                  entry.firewall_rule_id)
-                if not fwr_db['public']:
-                    raise fw_ext.FirewallPolicySharingConflict(
+                if not fwp_db['shared']:
+                    raise f_exc.FirewallPolicySharingConflict(
                         firewall_rule_id=fwr_db['id'],
                         firewall_policy_id=fwp_db['id'])
 
@@ -578,7 +585,7 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
                 filters=filters)
         for entry in fwg_with_fwp_id_db:
             if entry.tenant_id != fwp_tenant_id:
-                raise fw_ext.FirewallPolicyInUse(
+                raise f_exc.FirewallPolicyInUse(
                             firewall_policy_id=fwp_id)
 
     def _set_rules_for_policy(self, context, firewall_policy_db, fwp):
@@ -626,8 +633,8 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
                 tenant_id=fwp['tenant_id'],
                 name=fwp['name'],
                 description=fwp['description'],
-                public=fwp['public'],
-                audited=fwp['audited'])
+                audited=fwp['audited'],
+                shared=fwp['shared'])
             context.session.add(fwp_db)
             self._set_rules_for_policy(context, fwp_db, fwp)
         return self._make_firewall_policy_dict(fwp_db)
@@ -637,13 +644,13 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
         fwp = firewall_policy['firewall_policy']
         with context.session.begin(subtransactions=True):
             fwp_db = self._get_firewall_policy(context, id)
-            if not fwp.get('public', True):
-                # an update is setting public to False, make sure associated
+            if not fwp.get('shared', True):
+                # an update is setting shared to False, make sure associated
                 # firewall groups are in the same project.
                 self._check_fwgs_associated_with_policy_in_same_project(
                     context, id, fwp_db['tenant_id'])
-            if 'public' in fwp and 'firewall_rules' not in fwp:
-                self._check_if_rules_public_for_policy_public(
+            if 'shared' in fwp and 'firewall_rules' not in fwp:
+                self._check_if_rules_shared_for_policy_shared(
                     context, fwp_db, fwp)
             if 'firewall_rules' in fwp:
                 self._set_rules_for_policy(context, fwp_db, fwp)
@@ -660,9 +667,9 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
             # check if policy in use
             qry = context.session.query(FirewallGroup)
             if qry.filter_by(ingress_firewall_policy_id=id).first():
-                raise fw_ext.FirewallPolicyInUse(firewall_policy_id=id)
+                raise f_exc.FirewallPolicyInUse(firewall_policy_id=id)
             elif qry.filter_by(egress_firewall_policy_id=id).first():
-                raise fw_ext.FirewallPolicyInUse(firewall_policy_id=id)
+                raise f_exc.FirewallPolicyInUse(firewall_policy_id=id)
             else:
                 # Policy is not being used, delete.
                 self._delete_rules_in_policy(context, id)
@@ -685,16 +692,16 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
             fwp_id = fwg['ingress_firewall_policy_id']
             if fwp_id is not None:
                 fwp = self._get_firewall_policy(context, fwp_id)
-                if fwg_tenant_id != fwp['tenant_id'] and not fwp['public']:
-                    raise fw_ext.FirewallPolicyConflict(
+                if fwg_tenant_id != fwp['tenant_id'] and not fwp['shared']:
+                    raise f_exc.FirewallPolicyConflict(
                         firewall_policy_id=fwp_id)
 
         if 'egress_firewall_policy_id' in fwg:
             fwp_id = fwg['egress_firewall_policy_id']
             if fwp_id is not None:
                 fwp = self._get_firewall_policy(context, fwp_id)
-                if fwg_tenant_id != fwp['tenant_id'] and not fwp['public']:
-                    raise fw_ext.FirewallPolicyConflict(
+                if fwg_tenant_id != fwp['tenant_id'] and not fwp['shared']:
+                    raise f_exc.FirewallPolicyConflict(
                         firewall_policy_id=fwp_id)
         return
 
@@ -741,7 +748,7 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
             FirewallGroupPortAssociation.firewall_group_id != fwg_id).all()
         if fwg_ports:
             port_ids = [entry.port_id for entry in fwg_ports]
-            raise fw_ext.FirewallGroupPortInUse(port_ids=port_ids)
+            raise f_exc.FirewallGroupPortInUse(port_ids=port_ids)
 
     def create_firewall_group(self, context, firewall_group, status=None):
         fwg = firewall_group['firewall_group']
@@ -754,11 +761,11 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
                 tenant_id=fwg['tenant_id'],
                 name=fwg['name'],
                 description=fwg['description'],
-                public=fwg['public'],
                 status=status,
                 ingress_firewall_policy_id=fwg['ingress_firewall_policy_id'],
                 egress_firewall_policy_id=fwg['egress_firewall_policy_id'],
-                admin_state_up=fwg['admin_state_up'])
+                admin_state_up=fwg['admin_state_up'],
+                shared=fwg['shared'])
             context.session.add(fwg_db)
             self._set_ports_for_firewall_group(context, fwg_db, fwg)
         return self._make_firewall_group_dict(fwg_db)
@@ -777,7 +784,7 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
             count = context.session.query(
                 FirewallGroup).filter_by(id=id).update(fwg)
             if not count:
-                raise fw_ext.FirewallGroupNotFound(firewall_id=id)
+                raise f_exc.FirewallGroupNotFound(firewall_id=id)
         return self.get_firewall_group(context, id)
 
     def update_firewall_group_status(self, context, id, status, not_in=None):
@@ -801,7 +808,7 @@ class Firewall_db_mixin_v2(fw_ext.Firewallv2PluginBase, base_db.CommonDbMixin):
             count = context.session.query(
                 FirewallGroup).filter_by(id=id).delete()
             if not count:
-                raise fw_ext.FirewallGroupNotFound(firewall_id=id)
+                raise f_exc.FirewallGroupNotFound(firewall_id=id)
 
     def get_firewall_group(self, context, id, fields=None):
         LOG.debug("get_firewall_group() called")
