@@ -14,7 +14,10 @@
 #    under the License.
 
 from neutron.common import rpc as n_rpc
-from neutron import context
+from neutron_lib.agent import l3_extension
+from neutron_lib.api.definitions import firewall as fw_ext
+from neutron_lib import constants as nl_constants
+from neutron_lib import context
 from oslo_config import cfg
 from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
@@ -22,11 +25,8 @@ from oslo_log import log as logging
 from neutron_fwaas._i18n import _, _LE
 from neutron_fwaas.common import fwaas_constants
 from neutron_fwaas.common import resources as f_resources
-from neutron_fwaas.extensions import firewall as fw_ext
 from neutron_fwaas.services.firewall.agents import firewall_agent_api as api
 from neutron_fwaas.services.firewall.agents import firewall_service
-from neutron_lib.agent import l3_extension
-from neutron_lib import constants as nl_constants
 
 LOG = logging.getLogger(__name__)
 
@@ -64,7 +64,7 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
         self._register_rpc_consumers(connection)
 
     def consume_api(self, agent_api):
-        LOG.debug("FWaaS consume_api call occurred with %s" % agent_api)
+        LOG.debug("FWaaS consume_api call occurred with %s", agent_api)
         self.agent_api = agent_api
 
     def _register_rpc_consumers(self, connection):
@@ -394,9 +394,9 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                 router_ids,
                 firewall['tenant_id'])
             LOG.debug(
-                "Delete firewall %(fw)s on routers: '%(routers)s'"
-                % {'fw': firewall['id'],
-                   'routers': [ri.router['id'] for ri in router_info_list]})
+                "Delete firewall %(fw)s on routers: '%(routers)s'",
+                {'fw': firewall['id'],
+                 'routers': [ri.router['id'] for ri in router_info_list]})
             # call into the driver
             try:
                 self.fwaas_driver.delete_firewall(
