@@ -15,8 +15,9 @@
 
 from neutron.agent.linux import ip_lib
 from neutron.common import rpc as n_rpc
-from neutron import context
+from neutron_lib.agent import l3_extension
 from neutron_lib import constants as nl_constants
+from neutron_lib import context
 from oslo_config import cfg
 from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
@@ -27,7 +28,7 @@ from neutron_fwaas.common import resources as f_resources
 from neutron_fwaas.extensions import firewall as fw_ext
 from neutron_fwaas.services.firewall.agents import firewall_agent_api as api
 from neutron_fwaas.services.firewall.agents import firewall_service
-from neutron_lib.agent import l3_extension
+
 
 LOG = logging.getLogger(__name__)
 
@@ -80,7 +81,7 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
         self._register_rpc_consumers(connection)
 
     def consume_api(self, agent_api):
-        LOG.debug("FWaaS consume_api call occurred with %s" % agent_api)
+        LOG.debug("FWaaS consume_api call occurred with %s", agent_api)
         self.agent_api = agent_api
 
     def _register_rpc_consumers(self, connection):
@@ -165,8 +166,12 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
         return self._get_in_ns_ports(fwg_port_ids)
 
     def _get_in_ns_ports(self, port_ids):
-        """Returns port objects in the local namespace, along with their
-           router_info.
+        """Get ports in namespace by their IDs.
+
+        Returns port objects in the local namespace, along with their
+        router_info.
+
+        :param port_ids: IDs of router ports (set, list or tuple)
         """
         in_ns_ports = {}  # This will be converted to a list later.
         if port_ids and self.agent_api:
@@ -180,13 +185,21 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                         in_ns_ports[router_info] = [port_id]
         return list(in_ns_ports.items())
 
-    def _invoke_driver_for_sync_from_plugin(self, ctx, port, firewall_group):
-        """Calls the FWaaS driver's delete_firewall_group method if firewall
-           group has status of PENDING_DELETE; calls driver's
-           update_firewall_group method for all other statuses. Both of these
-           methods are idempotent.
+    def _invoke_driver_for_sync_from_plugin(self, ctx, ports, firewall_group):
+        """Call driver to sync firewall group.
+
+        Calls the FWaaS driver's delete_firewall_group method if firewall
+        group has status of PENDING_DELETE; calls driver's
+        update_firewall_group method for all other statuses. Both of these
+        methods are idempotent.
+
+        :param ctx: RPC context
+        :param ports: IDs of ports associated with a firewall group
+                      (set, list or tuple)
+        :param firewall_group: Dictionary describing the firewall group object
+
         """
-        port_list = self._get_in_ns_ports([port['id']])
+        port_list = self._get_in_ns_ports(ports)
         if firewall_group['status'] == nl_constants.PENDING_DELETE:
             try:
                 self.fwaas_driver.delete_firewall_group(
@@ -244,17 +257,31 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
         ctx = context.Context('', updated_router['tenant_id'])
         fwg_list = self.fwplugin_rpc.get_firewall_groups_for_project(ctx)
 
+        if nl_constants.INTERFACE_KEY not in updated_router:
+            return
+
         # Apply a firewall group, as requested, to ports on the new router.
-        if nl_constants.INTERFACE_KEY in updated_router:
-            for port in updated_router[nl_constants.INTERFACE_KEY]:
-                for firewall_group in fwg_list:
-                    if (self._has_port_insertion_fields(firewall_group) and
-                            (port['id'] in firewall_group['add-port-ids'] or
-                            port['id'] in firewall_group['del-port-ids'])):
-                        self._invoke_driver_for_sync_from_plugin(ctx, port,
-                                firewall_group)
-                        # A port can have at most one firewall group.
-                        break
+        all_router_ports = set(
+            p['id'] for p in updated_router[nl_constants.INTERFACE_KEY]
+        )
+        processed_ports = set()
+        for firewall_group in fwg_list:
+            if not self._has_port_insertion_fields(firewall_group):
+                continue
+
+            ports_to_process = (set(firewall_group['add-port-ids'] +
+                                    firewall_group['del-port-ids']) &
+                                all_router_ports)
+            # A port can have at most one firewall group.
+            port_ids_to_exclude = ports_to_process & processed_ports
+            if port_ids_to_exclude:
+                LOG.warning("Port(s) %s is associated with "
+                            "more than one firewall group(s).",
+                            port_ids_to_exclude)
+                ports_to_process -= port_ids_to_exclude
+            self._invoke_driver_for_sync_from_plugin(
+                ctx, ports_to_process, firewall_group)
+            processed_ports |= ports_to_process
 
     def add_router(self, context, new_router):
         """Handles agent restart and router add. Fetches firewall groups from
@@ -335,10 +362,10 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
         if not ports_for_fwg:
             return
 
-        LOG.debug("Create firewall group %(fwg_id)s on ports: %(ports)s"
-                % {'fwg_id': firewall_group['id'],
-                   'ports': ', '.join([p for ri_ports in ports_for_fwg
-                            for p in ri_ports[1]])})
+        LOG.debug("Create firewall group %(fwg_id)s on ports: %(ports)s",
+                 {'fwg_id': firewall_group['id'],
+                  'ports': ', '.join([p for ri_ports in ports_for_fwg
+                                      for p in ri_ports[1]])})
 
         # Set firewall group status; will be overwritten if call to driver
         # fails.
@@ -385,8 +412,9 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
         if ports_for_fwg:
             fw_ports = [p for ri_ports in ports_for_fwg for p in ri_ports[1]]
             LOG.debug("Update (delete) firewall group %(fwg_id)s on ports: "
-                    "%(ports)s" % {'fwg_id': firewall_group['id'],
-                    'ports': ', '.join(fw_ports)})
+                      "%(ports)s",
+                      {'fwg_id': firewall_group['id'],
+                       'ports': ', '.join(fw_ports)})
 
             # Set firewall group's status; will be overwritten if call to
             # driver fails.
@@ -418,8 +446,9 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                 fw_ports = [p for ri_ports in ports_for_fwg
                             for p in ri_ports[1]]
                 LOG.debug("Update (create) firewall group %(fwg_id)s on "
-                          "ports: %(ports)s" % {'fwg_id': firewall_group['id'],
-                                       'ports': ', '.join(fw_ports)})
+                          "ports: %(ports)s",
+                          {'fwg_id': firewall_group['id'],
+                           'ports': ', '.join(fw_ports)})
 
                 # Set firewall group status, which will be overwritten if call
                 # to driver fails.
@@ -463,8 +492,8 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
             return
 
         fw_ports = [p for ri_ports in ports_for_fwg for p in ri_ports[1]]
-        LOG.debug("Delete firewall group %(fwg_id)s on ports: %(ports)s"
-                % {'fwg_id': firewall_group['id'],
+        LOG.debug("Delete firewall group %(fwg_id)s on ports: %(ports)s",
+                  {'fwg_id': firewall_group['id'],
                    'ports': ', '.join(fw_ports)})
 
         # Set the firewall group's status to return to plugin; status may be
