@@ -32,7 +32,7 @@ class FWaaSClientMixin(object):
     @classmethod
     def resource_setup(cls):
         super(FWaaSClientMixin, cls).resource_setup()
-        manager = cls.manager
+        manager = cls.os_primary
         cls.firewall_groups_client = v2_client.FirewallGroupsClient(
             manager.auth_provider,
             CONF.network.catalog_type,
@@ -94,6 +94,26 @@ class FWaaSClientMixin(object):
                                         [nl_constants.PENDING_DELETE],
                                         not_found_ok=True)
 
+    def insert_firewall_rule_in_policy(self,
+                                       firewall_policy_id,
+                                       firewall_rule_id, **kwargs):
+        self.firewall_policies_client.insert_firewall_rule_in_policy(
+            firewall_policy_id=firewall_policy_id,
+            firewall_rule_id=firewall_rule_id,
+            **kwargs)
+        self.addCleanup(
+            test_utils.call_and_ignore_notfound_exc,
+            self.firewall_policies_client.remove_firewall_rule_from_policy,
+            firewall_policy_id=firewall_policy_id,
+            firewall_rule_id=firewall_rule_id)
+
+    def remove_firewall_rule_from_policy(self,
+                                         firewall_policy_id,
+                                         firewall_rule_id):
+        self.firewall_policies_client.remove_firewall_rule_from_policy(
+            firewall_policy_id=firewall_policy_id,
+            firewall_rule_id=firewall_rule_id)
+
     def _wait_firewall_group_ready(self, firewall_group_id):
         self._wait_firewall_group_while(firewall_group_id,
                                         [nl_constants.PENDING_CREATE,
