@@ -14,10 +14,13 @@
 #  under the License.
 
 import mock
-from neutron.api.v2 import attributes as attr
-from neutron import context
 from neutron.tests import fake_notifier
 from neutron.tests.unit.extensions import test_l3 as test_l3_plugin
+from neutron_lib import constants as nl_constants
+from neutron_lib import context
+from neutron_lib.exceptions import firewall_v2 as f_exc
+from neutron_lib.plugins import constants as plugin_constants
+from neutron_lib.plugins import directory
 from oslo_config import cfg
 import six
 
@@ -27,8 +30,7 @@ from neutron_fwaas.services.firewall import fwaas_plugin_v2
 from neutron_fwaas.tests import base
 from neutron_fwaas.tests.unit.db.firewall.v2 import (
     test_firewall_db_v2 as test_db_firewall)
-from neutron_lib import constants as nl_constants
-from neutron_lib.plugins import directory
+
 
 extensions_path = neutron_fwaas.extensions.__path__[0]
 
@@ -99,10 +101,6 @@ class TestFirewallRouterPortBase(
                      'TestL3NatServicePlugin')
 
         cfg.CONF.set_override('api_extensions_path', extensions_path)
-        self.saved_attr_map = {}
-        for resource, attrs in six.iteritems(attr.RESOURCE_ATTRIBUTE_MAP):
-            self.saved_attr_map[resource] = attrs.copy()
-        self.addCleanup(self.restore_attribute_map)
         if not fw_plugin:
             fw_plugin = FW_PLUGIN_KLASS
         service_plugins = {'l3_plugin_name': l3_plugin,
@@ -115,14 +113,10 @@ class TestFirewallRouterPortBase(
 
         self.setup_notification_driver()
 
-        self.l3_plugin = directory.get_plugin(nl_constants.L3)
+        self.l3_plugin = directory.get_plugin(plugin_constants.L3)
         self.plugin = directory.get_plugin('FIREWALL_V2')
         self.callbacks = self.plugin.endpoints[0]
 
-    def restore_attribute_map(self):
-        # Restore the original RESOURCE_ATTRIBUTE_MAP
-        attr.RESOURCE_ATTRIBUTE_MAP = self.saved_attr_map
-
 
 class TestFirewallCallbacks(TestFirewallRouterPortBase):
 
@@ -141,8 +135,7 @@ class TestFirewallCallbacks(TestFirewallRouterPortBase):
             ) as fwg:
                 fwg_id = fwg['firewall_group']['id']
                 res = self.callbacks.set_firewall_group_status(ctx, fwg_id,
-                                                         nl_constants.ACTIVE,
-                                                         host='dummy')
+                                                         nl_constants.ACTIVE)
                 fwg_db = self.plugin.get_firewall_group(ctx, fwg_id)
                 self.assertEqual(nl_constants.ACTIVE, fwg_db['status'])
                 self.assertTrue(res)
@@ -166,11 +159,10 @@ class TestFirewallCallbacks(TestFirewallRouterPortBase):
                     fwg_db = self.plugin._get_firewall_group(ctx, fwg_id)
                     fwg_db['status'] = nl_constants.PENDING_DELETE
 
-                observed = self.callbacks.firewall_group_deleted(
-                    ctx, fwg_id, host='dummy')
+                observed = self.callbacks.firewall_group_deleted(ctx, fwg_id)
                 self.assertTrue(observed)
 
-            self.assertRaises(firewall_v2.FirewallGroupNotFound,
+            self.assertRaises(f_exc.FirewallGroupNotFound,
                               self.plugin.get_firewall_group,
                               ctx, fwg_id)
 
@@ -203,17 +195,17 @@ class TestFirewallCallbacks(TestFirewallRouterPortBase):
                     self.plugin, '_get_firewall_group', side_effect=getdelete
                 ):
                     observed = self.callbacks.firewall_group_deleted(
-                        ctx, fwg_id, host='dummy')
+                        ctx, fwg_id)
                     self.assertTrue(observed)
 
-                self.assertRaises(firewall_v2.FirewallGroupNotFound,
+                self.assertRaises(f_exc.FirewallGroupNotFound,
                                   self.plugin.get_firewall_group,
                                   ctx, fwg_id)
 
     def test_firewall_group_deleted_not_found(self):
         ctx = context.get_admin_context()
         observed = self.callbacks.firewall_group_deleted(
-            ctx, 'notfound', host='hh')
+            ctx, 'notfound')
         self.assertTrue(observed)
 
     def test_firewall_group_deleted_error(self):
@@ -226,7 +218,7 @@ class TestFirewallCallbacks(TestFirewallRouterPortBase):
             ) as fwg:
                 fwg_id = fwg['firewall_group']['id']
                 observed = self.callbacks.firewall_group_deleted(
-                    ctx, fwg_id, host='dummy')
+                    ctx, fwg_id)
                 self.assertFalse(observed)
                 fwg_db = self.plugin._get_firewall_group(ctx, fwg_id)
                 self.assertEqual(nl_constants.ERROR, fwg_db['status'])
@@ -678,6 +670,34 @@ class TestFirewallPluginBasev2(TestFirewallRouterPortBase,
                                           s['subnet']['id'],
                                           None)
 
+    def test_update_firewall_group_with_invalid_project(self):
+        with self.router(name='router1',
+                         admin_state_up=True,
+                         tenant_id=self._tenant_id) as r, \
+                self.subnet(cidr='30.0.0.0/24') as s:
+            body = self._router_interface_action('add',
+                                                 r['router']['id'],
+                                                 s['subnet']['id'],
+                                                 None)
+            port_id = body['port_id']
+            with self.firewall_group(name='test',
+                                     default_policy=False,
+                                     ports=[],
+                                     admin_state_up=True) as fwg1:
+                data = {'firewall_group': {'ports': [port_id]}}
+                req = self.new_update_request(
+                    'firewall_groups', data, fwg1['firewall_group']['id'],
+                    context=context.get_admin_context())
+                res = self.deserialize(self.fmt,
+                                       req.get_response(self.ext_api))
+                self.assertEqual('FirewallGroupPortInvalidProject',
+                                 res['NeutronError']['type'])
+
+            self._router_interface_action('remove',
+                                          r['router']['id'],
+                                          s['subnet']['id'],
+                                          None)
+
     def test_update_firewall_group_with_ports_and_polcy(self):
         """neutron firewall_group create test-policy """
         with self.router(name='router1', admin_state_up=True,
